@@ -186,6 +186,9 @@ sub gimme_conf {
             $conf->set_program( make => $make );
         }
     }
+
+$conf->set_conf( source_engine =>  $ENV{CPANPLUS_SOURCE_ENGINE} )
+    if $ENV{CPANPLUS_SOURCE_ENGINE};
     
     _clean_test_dir( [
         $conf->get_conf('base'),     
